/* packet-rnsap.c
 * Routines for dissecting Universal Mobile Telecommunications System (UMTS);
 * UTRAN Iur interface Radio Network Subsystem
 * Application Part (RNSAP) signalling
 * (3GPP TS 25.423 version 6.7.0 Release 6) packet dissection
 * Copyright 2005, Anders Broman <anders.broman@ericsson.com>
 *
 * $Id$
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Ref: 3GPP TS 25.423 version 6.7.0 Release 6
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <glib.h>
#include <epan/packet.h>
#include <epan/conversation.h>

#include <stdio.h>
#include <string.h>

#include "packet-rnsap.h"
#include "packet-ber.h"
#include "packet-per.h"

#define PNAME  "UTRAN Iur interface Radio Network Subsystem Application Part"
#define PSNAME "RNSAP"
#define PFNAME "rnsap"

#define RNSAP_FDD 1
/* Procedure codes */
#define	RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATION			 0
#define	RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESRELEASE					 1
#define	RNSAP_ID_COMPRESSEDMODECOMMAND									 2
#define	RNSAP_ID_DOWNLINKPOWERCONTROL									 3
#define	RNSAP_ID_DOWNLINKPOWERTIMESLOTCONTROL							 4
#define	RNSAP_ID_DOWNLINKSIGNALLINGTRANSFER								 5
#define	RNSAP_ID_ERRORINDICATION										 6
#define	RNSAP_ID_DEDICATEDMEASUREMENTFAILURE							 7
#define	RNSAP_ID_DEDICATEDMEASUREMENTINITIATION							 8
#define	RNSAP_ID_DEDICATEDMEASUREMENTREPORTING							 9
#define	RNSAP_ID_DEDICATEDMEASUREMENTTERMINATION						 10
#define	RNSAP_ID_PAGING													 11
#define	RNSAP_ID_PHYSICALCHANNELRECONFIGURATION							 12
#define	RNSAP_ID_PRIVATEMESSAGE											 13
#define	RNSAP_ID_RADIOLINKADDITION										 14
#define	RNSAP_ID_RADIOLINKCONGESTION									 34
#define	RNSAP_ID_RADIOLINKDELETION										 15
#define	RNSAP_ID_RADIOLINKFAILURE										 16
#define	RNSAP_ID_RADIOLINKPREEMPTION									 17
#define	RNSAP_ID_RADIOLINKRESTORATION									 18
#define	RNSAP_ID_RADIOLINKSETUP											 19
#define	RNSAP_ID_RELOCATIONCOMMIT										 20
#define	RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCANCELLATION		 21
#define	RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCOMMIT				 22
#define	RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION		 23
#define	RNSAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION					 24
#define	RNSAP_ID_UPLINKSIGNALLINGTRANSFER								 25
#define	RNSAP_ID_COMMONMEASUREMENTFAILURE								 26
#define	RNSAP_ID_COMMONMEASUREMENTINITIATION							 27
#define	RNSAP_ID_COMMONMEASUREMENTREPORTING								 28
#define	RNSAP_ID_COMMONMEASUREMENTTERMINATION							 29
#define	RNSAP_ID_INFORMATIONEXCHANGEFAILURE								 30
#define	RNSAP_ID_INFORMATIONEXCHANGEINITIATION							 31
#define	RNSAP_ID_INFORMATIONREPORTING									 32
#define	RNSAP_ID_INFORMATIONEXCHANGETERMINATION							 33
#define	RNSAP_ID_RESET													 35
#define	RNSAP_ID_RADIOLINKACTIVATION 									 36
#define	RNSAP_ID_GERANUPLINKSIGNALLINGTRANSFER							 37
#define	RNSAP_ID_RADIOLINKPARAMETERUPDATE								 38
#define	RNSAP_ID_UEMEASUREMENTFAILURE									 39
#define	RNSAP_ID_UEMEASUREMENTINITIATION								 40
#define	RNSAP_ID_UEMEASUREMENTREPORTING									 41
#define	RNSAP_ID_UEMEASUREMENTTERMINATION								 42
#define	RNSAP_ID_IURDEACTIVATETRACE										 43
#define	RNSAP_ID_IURINVOKETRACE											 44
#define	RNSAP_ID_MBMSATTACH												 45
#define	RNSAP_ID_MBMSDETACH												 46
#define	RNSAP_ID_DIRECTINFORMATIONTRANSFER								 48

#define	RNSAP_ID_ALLOWEDQUEUINGTIME														 4
#define	RNSAP_ID_ALLOWED_RATE_INFORMATION												 42
#define	RNSAP_ID_ANTENNACOLOCATIONINDICATOR												 309
#define	RNSAP_ID_BINDINGID																 5
#define	RNSAP_ID_C_ID																	 6
#define	RNSAP_ID_C_RNTI																	 7
#define	RNSAP_ID_CELL_CAPACITY_CLASS_VALUE												 303
#define	RNSAP_ID_CFN																	 8
#define	RNSAP_ID_CN_CS_DOMAINIDENTIFIER													 9
#define	RNSAP_ID_CN_PS_DOMAINIDENTIFIER													 10
#define	RNSAP_ID_CAUSE																	 11
#define	RNSAP_ID_COVERAGEINDICATOR														 310
#define	RNSAP_ID_CRITICALITYDIAGNOSTICS													 20
#define	RNSAP_ID_CONTEXTINFOITEM_RESET													 211
#define	RNSAP_ID_CONTEXTGROUPINFOITEM_RESET												 515
#define	RNSAP_ID_D_RNTI																	 21
#define	RNSAP_ID_D_RNTI_RELEASEINDICATION												 22
#define	RNSAP_ID_DCHS_TO_ADD_FDD														 26
#define	RNSAP_ID_DCHS_TO_ADD_TDD														 27
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPFDD										 30
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPTDD										 31
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTFDD										 32
#define	RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTTDD										 33
#define	RNSAP_ID_DCH_FDD_INFORMATION													 34
#define	RNSAP_ID_DCH_TDD_INFORMATION													 35
#define	RNSAP_ID_FDD_DCHS_TO_MODIFY														 39
#define	RNSAP_ID_TDD_DCHS_TO_MODIFY														 40
#define	RNSAP_ID_DCH_INFORMATIONRESPONSE												 43
#define	RNSAP_ID_DCH_RATE_INFORMATIONITEM_RL_CONGESTIND									 38
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONADDITEM_RL_RECONFPREPTDD							 44
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD							 45
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD						 46
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD								 47
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD							 48
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD							 49
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD								 50
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD							 51
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD						 52
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD								 53
#define	RNSAP_ID_FDD_DL_CODEINFORMATION													 54
#define	RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFPREPFDD									 59
#define	RNSAP_ID_DL_DPCH_INFORMATION_RL_SETUPRQSTFDD									 60
#define	RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFRQSTFDD									 61
#define	RNSAP_ID_DL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD								 62
#define	RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD								 63
#define	RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD									 64
#define	RNSAP_ID_DL_DPCH_TIMINGADJUSTMENT												 278
#define	RNSAP_ID_DLREFERENCEPOWER														 67
#define	RNSAP_ID_DLREFERENCEPOWERLIST_DL_PC_RQST										 68
#define	RNSAP_ID_DL_REFERENCEPOWERINFORMATION_DL_PC_RQST								 69
#define	RNSAP_ID_DPC_MODE																 12
#define	RNSAP_ID_DRXCYCLELENGTHCOEFFICIENT												 70
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL_IND								 470
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL									 471
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RPRT									 71
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RQST									 72
#define	RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RSP									 73
#define	RNSAP_ID_DEDICATEDMEASUREMENTTYPE												 74
#define	RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPFDD						 82
#define	RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPTDD						 83
#define	RNSAP_ID_GUARANTEED_RATE_INFORMATION											 41
#define	RNSAP_ID_IMSI																	 84
#define	RNSAP_ID_HCS_PRIO																 311
#define	RNSAP_ID_L3_INFORMATION															 85
#define	RNSAP_ID_ADJUSTMENTPERIOD														 90
#define	RNSAP_ID_MAXADJUSTMENTSTEP														 91
#define	RNSAP_ID_MEASUREMENTFILTERCOEFFICIENT											 92
#define	RNSAP_ID_MESSAGESTRUCTURE														 57
#define	RNSAP_ID_MEASUREMENTID															 93
#define	RNSAP_ID_NEIGHBOURING_GSM_CELLINFORMATION										 13
#define	RNSAP_ID_NEIGHBOURING_UMTS_CELLINFORMATIONITEM									 95
#define	RNSAP_ID_NRT_LOAD_INFORMATION_VALUE												 305
#define	RNSAP_ID_NRT_LOAD_INFORMATION_VALUE_INCRDECRTHRES								 306
#define	RNSAP_ID_PAGINGAREA_PAGINGRQST													 102
#define	RNSAP_ID_FACH_FLOWCONTROLINFORMATION											 103
#define	RNSAP_ID_PARTIALREPORTINGINDICATOR												 472
#define	RNSAP_ID_PERMANENT_NAS_UE_IDENTITY												 17
#define	RNSAP_ID_POWERADJUSTMENTTYPE													 107
#define	RNSAP_ID_RANAP_RELOCATIONINFORMATION											 109
#define	RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTFDD										 110
#define	RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTTDD										 111
#define	RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTFDD										 112
#define	RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTTDD										 113
#define	RNSAP_ID_RL_INFORMATION_RL_DELETIONRQST											 114
#define	RNSAP_ID_RL_INFORMATION_RL_FAILUREIND											 115
#define	RNSAP_ID_RL_INFORMATION_RL_RECONFPREPFDD										 116
#define	RNSAP_ID_RL_INFORMATION_RL_RESTOREIND											 117
#define	RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTFDD											 118
#define	RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTTDD											 119
#define	RNSAP_ID_RL_INFORMATIONITEM_RL_CONGESTIND										 55
#define	RNSAP_ID_RL_INFORMATIONITEM_DM_RPRT												 120
#define	RNSAP_ID_RL_INFORMATIONITEM_DM_RQST												 121
#define	RNSAP_ID_RL_INFORMATIONITEM_DM_RSP												 122
#define	RNSAP_ID_RL_INFORMATIONITEM_RL_PREEMPTREQUIREDIND								 2
#define	RNSAP_ID_RL_INFORMATIONITEM_RL_SETUPRQSTFDD										 123
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_CONGESTIND										 56
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_ADDITIONRQSTFDD									 124
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_DELETIONRQST										 125
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_PREEMPTREQUIREDIND								 1
#define	RNSAP_ID_RL_INFORMATIONLIST_RL_RECONFPREPFDD									 126
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD								 127
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD								 128
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_SETUPRSPTDD									 129
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_ADDITIONRSPFDD							 130
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFREADYFDD							 131
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFRSPFDD								 132
#define	RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_SETUPRSPFDD								 133
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_ADDITIONRSPFDD							 134
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFREADYFDD							 135
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFRSPFDD								 136
#define	RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFRSPTDD									 28
#define	RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_SETUPRSPFDD								 137
#define	RNSAP_ID_RL_RECONFIGURATIONFAILURE_RL_RECONFFAIL								 141
#define	RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RPRT											 143
#define	RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RQST											 144
#define	RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RSP											 145
#define	RNSAP_ID_RL_SET_INFORMATION_RL_FAILUREIND										 146
#define	RNSAP_ID_RL_SET_INFORMATION_RL_RESTOREIND										 147
#define	RNSAP_ID_RL_SET_SUCCESSFUL_INFORMATIONITEM_DM_FAIL								 473
#define	RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL							 474
#define	RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND						 475
#define	RNSAP_ID_RL_SUCCESSFUL_INFORMATIONITEM_DM_FAIL									 476
#define	RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL								 477
#define	RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND							 478
#define	RNSAP_ID_REPORTCHARACTERISTICS													 152
#define	RNSAP_ID_REPORTING_OBJECT_RL_FAILUREIND											 153
#define	RNSAP_ID_REPORING_OBJECT_RL_RESTOREIND											 154
#define	RNSAP_ID_RT_LOAD_VALUE															 307
#define	RNSAP_ID_RT_LOAD_VALUE_INCRDECRTHRES											 308
#define	RNSAP_ID_S_RNTI																	 155
#define	RNSAP_ID_RESETINDICATOR															 244
#define	RNSAP_ID_RNC_ID																	 245
#define	RNSAP_ID_SAI																	 156
#define	RNSAP_ID_SRNC_ID																 157
#define	RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD					 159
#define	RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD					 160
#define	RNSAP_ID_TRANSPORTBEARERID														 163
#define	RNSAP_ID_TRANSPORTBEARERREQUESTINDICATOR										 164
#define	RNSAP_ID_TRANSPORTLAYERADDRESS													 165
#define	RNSAP_ID_TYPEOFERROR															 140
#define	RNSAP_ID_UC_ID																	 166
#define	RNSAP_ID_UL_CCTRCH_ADDINFORMATION_RL_RECONFPREPTDD								 167
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD							 169
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD								 171
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD								 172
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD							 173
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD							 174
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD							 175
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD								 176
#define	RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFPREPFDD									 177
#define	RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFRQSTFDD									 178
#define	RNSAP_ID_UL_DPCH_INFORMATION_RL_SETUPRQSTFDD									 179
#define	RNSAP_ID_UL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD								 180
#define	RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD								 181
#define	RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD									 182
#define	RNSAP_ID_UL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD							 183
#define	RNSAP_ID_UL_SIRTARGET															 184
#define	RNSAP_ID_URA_INFORMATION														 185
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD				 188
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD					 189
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILURETDD					 190
#define	RNSAP_ID_ACTIVE_PATTERN_SEQUENCE_INFORMATION									 193
#define	RNSAP_ID_ADJUSTMENTRATIO														 194
#define	RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILUREFDD										 197
#define	RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILURETDD										 198
#define	RNSAP_ID_CAUSELEVEL_RL_RECONFFAILURE											 199
#define	RNSAP_ID_CAUSELEVEL_RL_SETUPFAILUREFDD											 200
#define	RNSAP_ID_CAUSELEVEL_RL_SETUPFAILURETDD											 201
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFPREPTDD						 205
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFPREPTDD						 206
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD						 207
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD						 208
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD						 209
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD						 210
#define	RNSAP_ID_DL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD							 212
#define	RNSAP_ID_DL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD						 213
#define	RNSAP_ID_DL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD						 214
#define	RNSAP_ID_DSCHS_TO_ADD_TDD														 215
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_216											 216
#define	RNSAP_ID_DSCH_DELETELIST_RL_RECONFPREPTDD										 217
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_218											 218
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_219											 219
#define	RNSAP_ID_DSCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD								 220
#define	RNSAP_ID_DSCH_INFORMATIONLISTIES_RL_SETUPRSPTDD									 221
#define	RNSAP_ID_DSCH_TDD_INFORMATION													 222
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_223											 223
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_226											 226
#define	RNSAP_ID_DSCH_MODIFYLIST_RL_RECONFPREPTDD										 227
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_228											 228
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_324											 324
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_229											 229
#define	RNSAP_ID_DSCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD							 230
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_29											 29
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_225											 225
#define	RNSAP_ID_GA_CELL																 232
#define	RNSAP_ID_GA_CELLADDITIONALSHAPES												 3
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_246											 246
#define	RNSAP_ID_TRANSMISSION_GAP_PATTERN_SEQUENCE_INFORMATION							 255
#define	RNSAP_ID_UL_CCTRCH_DELETEINFORMATION_RL_RECONFPREPTDD							 256
#define	RNSAP_ID_UL_CCTRCH_MODIFYINFORMATION_RL_RECONFPREPTDD							 257
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD						 258
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD						 259
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD						 260
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD						 261
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD						 262
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD						 263
#define	RNSAP_ID_UL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD						 264
#define	RNSAP_ID_UL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD						 265
#define	RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILURETDD				 266
#define	RNSAP_ID_USCHS_TO_ADD															 267
#define	RNSAP_ID_USCH_DELETELIST_RL_RECONFPREPTDD										 268
#define	RNSAP_ID_USCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD								 269
#define	RNSAP_ID_USCH_INFORMATIONLISTIES_RL_SETUPRSPTDD									 270
#define	RNSAP_ID_USCH_INFORMATION														 271
#define	RNSAP_ID_USCH_MODIFYLIST_RL_RECONFPREPTDD										 272
#define	RNSAP_ID_USCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD							 273
#define	RNSAP_ID_DL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD						 274
#define	RNSAP_ID_UL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD						 275
#define	RNSAP_ID_CLOSEDLOOPMODE1_SUPPORTINDICATOR										 276
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_277											 277
#define	RNSAP_ID_STTD_SUPPORTINDICATOR													 279
#define	RNSAP_ID_CFNREPORTINGINDICATOR             										 14
#define	RNSAP_ID_CNORIGINATEDPAGE_PAGINGRQST											 23
#define	RNSAP_ID_INNERLOOPDLPCSTATUS													 24
#define	RNSAP_ID_PROPAGATIONDELAY														 25
#define	RNSAP_ID_RXTIMINGDEVIATIONFORTA													 36
#define	RNSAP_ID_TIMESLOT_ISCP															 37
#define	RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_FAILUREIND									 15
#define	RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_RESTOREIND									 16
#define	RNSAP_ID_COMMONMEASUREMENTACCURACY												 280
#define	RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RPRT									 281
#define	RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RQST									 282
#define	RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RSP										 283
#define	RNSAP_ID_COMMONMEASUREMENTTYPE													 284
#define	RNSAP_ID_CONGESTIONCAUSE 														 18
#define	RNSAP_ID_SFN																	 285
#define	RNSAP_ID_SFNREPORTINGINDICATOR													 286
#define	RNSAP_ID_INFORMATIONEXCHANGEID													 287
#define	RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RPRT								 288
#define	RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RQST								 289
#define	RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RSP								 290
#define	RNSAP_ID_INFORMATIONREPORTCHARACTERISTICS										 291
#define	RNSAP_ID_INFORMATIONTYPE														 292
#define	RNSAP_ID_NEIGHBOURING_LCR_TDD_CELLINFORMATION									 58
#define	RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_SETUPRQSTTDD						 65
#define	RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_SETUPRSPTDD								 66
#define	RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD							 75
#define	RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD								 76
#define	RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD							 77
#define	RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD								 78
#define	RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD								 79
#define	RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD								 80
#define	RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_ADDITIONRQSTTDD					 81
#define	RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD							 86
#define	RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD						 87
#define	RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD							 88
#define	RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD						 89
#define	RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD							 94
#define	RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD							 96
#define	RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD							 97
#define	RNSAP_ID_UL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD						 98
#define	RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD				 100
#define	RNSAP_ID_DL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD						 101
#define	RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD				 104
#define	RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD						 105
#define	RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD						 106
#define	RNSAP_ID_TIMESLOT_ISCP_LCR_LIST_DL_PC_RQST_TDD									 138
#define	RNSAP_ID_TSTD_SUPPORT_INDICATOR_RL_SETUPRQSTTDD									 139
#define	RNSAP_ID_RESTRICTIONSTATEINDICATOR												 142
#define	RNSAP_ID_LOAD_VALUE																 233
#define	RNSAP_ID_LOAD_VALUE_INCRDECRTHRES												 234
#define	RNSAP_ID_ONMODIFICATION															 235
#define	RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE									 236
#define	RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE_INCRDECRTHRES						 237
#define	RNSAP_ID_SFNSFNMEASUREMENTTHRESHOLDINFORMATION									 238
#define	RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE										 239
#define	RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE_INCRDECRTHRES							 240
#define	RNSAP_ID_TUTRANGPSMEASUREMENTTHRESHOLDINFORMATION								 241
#define	RNSAP_ID_UL_TIMESLOT_ISCP_VALUE													 242
#define	RNSAP_ID_UL_TIMESLOT_ISCP_VALUE_INCRDECRTHRES									 243
#define	RNSAP_ID_RX_TIMING_DEVIATION_VALUE_LCR											 293
#define	RNSAP_ID_DPC_MODE_CHANGE_SUPPORTINDICATOR										 19
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_247											 247
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_295											 295
#define	RNSAP_ID_PRIMARYCCPCH_RSCP_RL_RECONFPREPTDD										 202
#define	RNSAP_ID_DL_TIMESLOT_ISCP_INFO_RL_RECONFPREPTDD									 203
#define	RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_RECONFPREPTDD						 204
#define	RNSAP_ID_DSCH_RNTI																 249
#define	RNSAP_ID_DL_POWERBALANCING_INFORMATION											 296
#define	RNSAP_ID_DL_POWERBALANCING_ACTIVATIONINDICATOR									 297
#define	RNSAP_ID_DL_POWERBALANCING_UPDATEDINDICATOR										 298
#define	RNSAP_ID_DL_REFERENCEPOWERINFORMATION											 299
#define	RNSAP_ID_ENHANCED_PRIMARYCPICH_ECNO												 224
#define	RNSAP_ID_IPDL_TDD_PARAMETERSLCR													 252
#define	RNSAP_ID_CELLCAPABILITYCONTAINER_FDD											 300
#define	RNSAP_ID_CELLCAPABILITYCONTAINER_TDD											 301
#define	RNSAP_ID_CELLCAPABILITYCONTAINER_TDD_LCR										 302
#define	RNSAP_ID_RL_SPECIFIC_DCH_INFO													 317
#define	RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATIONLIST 						 318
#define	RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATION_IES						 319
#define	RNSAP_ID_RL_RECONFIGURATIONREQUESTTDD_RL_INFORMATION							 321
#define	RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATIONNOTREQUIRED				 250
#define	RNSAP_ID_DELAYEDACTIVATION 														 312
#define	RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDFDD								 313
#define	RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDFDD						 314
#define	RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDTDD								 315
#define	RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDTDD						 316
#define	RNSAP_ID_NEIGHBOURINGTDDCELLMEASUREMENTINFORMATIONLCR							 251
#define	RNSAP_ID_UL_SIR_TARGET_CCTRCH_INFORMATIONITEM_RL_SETUPRSPTDD					 150
#define	RNSAP_ID_UL_SIR_TARGET_CCTRCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD				 151
#define	RNSAP_ID_PRIMCCPCH_RSCP_DL_PC_RQSTTDD											 451
#define	RNSAP_ID_HSDSCH_FDD_INFORMATION													 452
#define	RNSAP_ID_HSDSCH_FDD_INFORMATION_RESPONSE										 453
#define	RNSAP_ID_HSDSCH_FDD_UPDATE_INFORMATION											 466
#define	RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY											 456
#define	RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND			 516
#define	RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND			 517
#define	RNSAP_ID_HSDSCH_RNTI															 457
#define	RNSAP_ID_HSDSCH_TDD_INFORMATION													 458
#define	RNSAP_ID_HSDSCH_TDD_INFORMATION_RESPONSE										 459
#define	RNSAP_ID_HSDSCH_TDD_UPDATE_INFORMATION											 467
#define	RNSAP_ID_HSPDSCH_RL_ID															 463
#define	RNSAP_ID_HSDSCH_MACDFLOWS_TO_ADD 												 531
#define	RNSAP_ID_HSDSCH_MACDFLOWS_TO_DELETE												 532
#define	RNSAP_ID_ANGLE_OF_ARRIVAL_VALUE_LCR												 148
#define	RNSAP_ID_TRAFFICCLASS															 158
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_248											 248
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_253											 253
#define	RNSAP_ID_PDSCH_RL_ID															 323
#define	RNSAP_ID_TIMESLOT_RL_SETUPRSPTDD												 325
#define	RNSAP_ID_GERAN_CELL_CAPABILITY													 468
#define	RNSAP_ID_GERAN_CLASSMARK														 469
#define	RNSAP_ID_DSCH_INITIALWINDOWSIZE													 480
#define	RNSAP_ID_UL_SYNCHRONISATION_PARAMETERS_LCR										 464
#define	RNSAP_ID_SNA_INFORMATION														 479
#define	RNSAP_ID_MACHS_RESETINDICATOR													 465
#define	RNSAP_ID_TDD_DL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD				 481
#define	RNSAP_ID_TDD_UL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD				 482
#define	RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_SETUPRQSTTDD								 483
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD							 484
#define	RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD							 485
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD							 486
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD							 487
#define	RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONADD_LCR_RL_RECONFPREPTDD				 488
#define	RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONMODIFY_LCR_RL_RECONFPREPTDD			 489
#define	RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONADD_RL_RECONFPREPTDD				 490
#define	RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONMODIFY_RL_RECONFPREPTDD			 491
#define	RNSAP_ID_UL_TIMINGADVANCECTRL_LCR												 492
#define	RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLIST_PHYCHRECONFRQSTTDD					 493
#define	RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLISTLCR_PHYCHRECONFRQSTTDD					 494
#define	RNSAP_ID_HS_SICH_RECEPTION_QUALITY 												 495
#define	RNSAP_ID_HS_SICH_RECEPTION_QUALITY_MEASUREMENT_VALUE 							 496
#define	RNSAP_ID_HSSICH_INFO_DM_RPRT													 497
#define	RNSAP_ID_HSSICH_INFO_DM_RQST													 498
#define	RNSAP_ID_HSSICH_INFO_DM															 499
#define	RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_SETUPRSPTDD									 500
#define	RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_SETUPRSPTDD									 501
#define	RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_ADDITIONRSPTDD								 502
#define	RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_ADDITIONRSPTDD								 503
#define	RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_RECONFREADYTDD								 504
#define	RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_RECONFREADYTDD								 505
#define	RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD	 506
#define	RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD	 507
#define	RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_RECONFRSPTDD								 508
#define	RNSAP_ID_DL_DPCH_INFORMATIONMODIFYITEM_LCR_RL_RECONFRSPTDD						 509
#define	RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM							 510
#define	RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM							 511
#define	RNSAP_ID_TDD_SUPPORT_8PSK														 512
#define	RNSAP_ID_TDD_MAXNRDLPHYSICALCHANNELS											 513
#define	RNSAP_ID_EXTENDEDGSMCELLINDIVIDUALOFFSET										 514
#define	RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATIONLIST 					 518
#define	RNSAP_ID_PRIMARY_CPICH_USAGE_FOR_CHANNEL_ESTIMATION								 519
#define	RNSAP_ID_SECONDARY_CPICH_INFORMATION											 520
#define	RNSAP_ID_SECONDARY_CPICH_INFORMATION_CHANGE										 521
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_522											 522
#define	RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_523											 523
#define	RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATION_ITEM					 524
#define	RNSAP_ID_PHASE_REFERENCE_UPDATE_INDICATOR										 525
#define	RNSAP_ID_UNIDIRECTIONAL_DCH_INDICATOR											 526
#define	RNSAP_ID_RL_INFORMATION_RL_RECONFPREPTDD										 527
#define	RNSAP_ID_MULTIPLE_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD						 528
#define	RNSAP_ID_RL_RECONFIGURATIONRESPONSETDD_RL_INFORMATION							 529
#define	RNSAP_ID_SATELLITE_ALMANAC_INFORMATION_EXTITEM									 530
#define	RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY_UNSYNCHRONISED							 533
#define	RNSAP_ID_TNLQOS																	 534
#define	RNSAP_ID_RTLOADVALUE															 535
#define	RNSAP_ID_NRTLOADINFORMATIONVALUE												 536
#define	RNSAP_ID_CELLPORTIONID															 537
#define	RNSAP_ID_UPPTSINTERFERENCEVALUE													 538
#define	RNSAP_ID_PRIMARYCCPCH_RSCP_DELTA												 539
#define	RNSAP_ID_UEMEASUREMENTTYPE														 540
#define	RNSAP_ID_UEMEASUREMENTTIMESLOTINFOHCR											 541
#define	RNSAP_ID_UEMEASUREMENTTIMESLOTINFOLCR											 542
#define	RNSAP_ID_UEMEASUREMENTREPORTCHARACTERISTICS										 543
#define	RNSAP_ID_UEMEASUREMENTPARAMETERMODALLOW											 544
#define	RNSAP_ID_UEMEASUREMENTVALUEINFORMATION											 545
#define	RNSAP_ID_INTERFACESTOTRACEITEM													 546
#define	RNSAP_ID_LISTOFINTERFACESTOTRACE												 547
#define	RNSAP_ID_TRACEDEPTH																 548
#define	RNSAP_ID_TRACERECORDINGSESSIONREFERENCE											 549
#define	RNSAP_ID_TRACEREFERENCE															 550
#define	RNSAP_ID_UEIDENTITY																 551
#define	RNSAP_ID_NACC_RELATED_DATA														 552
#define	RNSAP_ID_GSM_CELL_INFEX_RQST													 553
#define	RNSAP_ID_MEASUREMENTRECOVERYBEHAVIOR											 554
#define	RNSAP_ID_MEASUREMENTRECOVERYREPORTINGINDICATOR									 555
#define	RNSAP_ID_MEASUREMENTRECOVERYSUPPORTINDICATOR									 556
#define	RNSAP_ID_DL_DPCH_POWER_INFORMATION_RL_RECONFPREPFDD								 557
#define	RNSAP_ID_F_DPCH_INFORMATION_RL_RECONFPREPFDD									 558
#define	RNSAP_ID_F_DPCH_INFORMATION_RL_SETUPRQSTFDD										 559
#define	RNSAP_ID_MBMS_BEARER_SERVICE_LIST												 560
#define	RNSAP_ID_MBMS_BEARER_SERVICE_LIST_INFEX_RSP										 561
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD											 562
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD											 563
#define	RNSAP_ID_OLD_URA_ID																 564
#define	RNSAP_ID_UE_STATE																 568
#define	RNSAP_ID_URA_ID																	 569
#define	RNSAP_ID_HARQ_PREAMBLE_MODE														 571
#define	RNSAP_ID_SYNCHRONISATIONINDICATOR												 572
#define	RNSAP_ID_UL_DPDCHINDICATOREDCH													 573
#define	RNSAP_ID_EDPCH_INFORMATION														 574
#define	RNSAP_ID_RL_SPECIFIC_EDCH_INFORMATION											 575
#define	RNSAP_ID_EDCH_RL_INDICATION														 576
#define	RNSAP_ID_EDCH_FDD_INFORMATION													 577
#define	RNSAP_ID_EDCH_RLSET_ID															 578
#define	RNSAP_ID_SERVING_EDCHRL_ID														 579
#define	RNSAP_ID_EDCH_FDD_DL_CONTROLCHANNELINFORMATION									 580
#define	RNSAP_ID_EDCH_FDD_INFORMATIONRESPONSE											 581
#define	RNSAP_ID_EDCH_MACDFLOWS_TO_ADD													 582
#define	RNSAP_ID_EDCH_FDD_INFORMATION_TO_MODIFY											 583
#define	RNSAP_ID_EDCH_MACDFLOWS_TO_DELETE												 584
#define	RNSAP_ID_EDPCH_INFORMATION_RLRECONFREQUEST_FDD									 585
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND				 586
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND				 587
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_CONGESTIND  					 588
#define	RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_CONGESTIND						 589
#define	RNSAP_ID_MBMS_BEARER_SERVICE_FULL_ADDRESS										 590
#define	RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT										 591
#define	RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT_ALLOWED								 592
#define	RNSAP_ID_USER_PLANE_CONGESTION_FIELDS_INCLUSION									 593
#define	RNSAP_ID_HARQ_PREAMBLE_MODE_ACTIVATION_INDICATOR								 594
#define	RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_TDD_DM_RSP						 595
#define	RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_LCR_TDD_DM_RSP					 596
#define	RNSAP_ID_PROVIDEDINFORMATION													 597
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD_PFL										 598
#define	RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD_PFL										 599
#define	RNSAP_ID_FREQUENCYBANDINDICATOR													 600
static dissector_handle_t rnsap_handle=NULL;

/* Initialize the protocol and registered fields */
static int proto_rnsap = -1;

static int hf_rnsap_pdu_length = -1;
static int hf_rnsap_IE_length = -1;

#include "packet-rnsap-hf.c"

/* Initialize the subtree pointers */
static int ett_rnsap = -1;
static int ett_rnsap_initiatingMessageValue = -1;
static int ett_rnsap_ProtocolIEValueValue = -1;
static int ett_rnsap_SuccessfulOutcomeValue = -1;
static int ett_rnsap_UnsuccessfulOutcomeValue = -1;

#include "packet-rnsap-ett.c"

/* Global variables */
static proto_tree *top_tree;
static guint32 ProcedureCode;
static guint32 ProtocolIE_ID;
static guint32 ddMode;

#define BYTE_ALIGN_OFFSET(offset)		\
	if(offset&0x07){			\
		offset=(offset&0xfffffff8)+8;	\
	}
#define RNSAP_FDD 1
/* Prodedure ID:s */

/* Protocol IE:s */



static int dissect_rnsap_InitiatingMessageValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree);
static int dissect_rnsap_SuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree);
static int dissect_rnsap_UnsuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree);
static int dissect_rnsap_ProtocolIEValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree);
#include "packet-rnsap-fn.c"


static int dissect_rnsap_InitiatingMessageValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_rnsap_initiatingMessageValue, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_rnsap_initiatingMessageValue);

	offset = dissect_per_length_determinant(tvb, offset, pinfo, value_tree, hf_rnsap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATION:	/* 0 */
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESRELEASE:			/* 1 */
	case RNSAP_ID_COMPRESSEDMODECOMMAND:							 /* 2 */
	case RNSAP_ID_DOWNLINKPOWERCONTROL:								 /* 3 */
	case RNSAP_ID_DOWNLINKPOWERTIMESLOTCONTROL:						 /* 4 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DOWNLINKSIGNALLINGTRANSFER:						 /* 5 */
		offset = dissect_id_downlinkSignallingTransfer(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_ERRORINDICATION:									 /* 6 */
	case RNSAP_ID_DEDICATEDMEASUREMENTFAILURE:						 /* 7 */
	case RNSAP_ID_DEDICATEDMEASUREMENTINITIATION:					 /* 8 */
	case RNSAP_ID_DEDICATEDMEASUREMENTREPORTING:					 /* 9 */
	case RNSAP_ID_DEDICATEDMEASUREMENTTERMINATION:					 /* 10 */
	case RNSAP_ID_PAGING:											 /* 11 */
	case RNSAP_ID_PHYSICALCHANNELRECONFIGURATION:					 /* 12 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_PRIVATEMESSAGE:									 /* 13 */
		offset = dissect_id_privateMessage(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_RADIOLINKADDITION:								 /* 14 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkAddition(tvb, offset, pinfo, value_tree);
		}else{
			offset = dissect_id_radioLinkAddition_TDD(tvb, offset, pinfo, value_tree);
		}
		break;
	case RNSAP_ID_RADIOLINKCONGESTION:								 /* 34 */
	case RNSAP_ID_RADIOLINKDELETION:								 /* 15 */
	case RNSAP_ID_RADIOLINKFAILURE:									 /* 16 */
	case RNSAP_ID_RADIOLINKPREEMPTION:								 /* 17 */
	case RNSAP_ID_RADIOLINKRESTORATION:								 /* 18 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_RADIOLINKSETUP:									 /* 19 */
		if (ddMode==RNSAP_FDD){
			offset = dissect_id_radioLinkSetup(tvb, offset, pinfo, value_tree);
		}else{
			offset = dissect_id_radioLinkSetupTdd(tvb, offset, pinfo, value_tree);
		}
		break;
	case RNSAP_ID_RELOCATIONCOMMIT:									 /* 20 */
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCANCELLATION:	 /* 21 */
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONCOMMIT:		 /* 22 */
	case RNSAP_ID_SYNCHRONISEDRADIOLINKRECONFIGURATIONPREPARATION:	 /* 23 */
	case RNSAP_ID_UNSYNCHRONISEDRADIOLINKRECONFIGURATION:			 /* 24 */
	case RNSAP_ID_UPLINKSIGNALLINGTRANSFER:							 /* 25 */
	case RNSAP_ID_COMMONMEASUREMENTFAILURE:							 /* 26 */
	case RNSAP_ID_COMMONMEASUREMENTINITIATION:						 /* 27 */
	case RNSAP_ID_COMMONMEASUREMENTREPORTING:						 /* 28 */
	case RNSAP_ID_COMMONMEASUREMENTTERMINATION:						 /* 29 */
	case RNSAP_ID_INFORMATIONEXCHANGEFAILURE:						 /* 30 */
	case RNSAP_ID_INFORMATIONEXCHANGEINITIATION:					 /* 31 */
	case RNSAP_ID_INFORMATIONREPORTING:								 /* 32 */
	case RNSAP_ID_INFORMATIONEXCHANGETERMINATION:					 /* 33 */
	case RNSAP_ID_RESET:											 /* 35 */
	case RNSAP_ID_RADIOLINKACTIVATION: 								 /* 36 */
	case RNSAP_ID_GERANUPLINKSIGNALLINGTRANSFER:					 /* 37 */
	case RNSAP_ID_RADIOLINKPARAMETERUPDATE:							 /* 38 */
	case RNSAP_ID_UEMEASUREMENTFAILURE:								 /* 39 */
	case RNSAP_ID_UEMEASUREMENTINITIATION:							 /* 40 */
	case RNSAP_ID_UEMEASUREMENTREPORTING:							 /* 41 */
	case RNSAP_ID_UEMEASUREMENTTERMINATION:							 /* 42 */
	case RNSAP_ID_IURDEACTIVATETRACE:								 /* 43 */
	case RNSAP_ID_IURINVOKETRACE:									 /* 44 */
	case RNSAP_ID_MBMSATTACH:										 /* 45 */
	case RNSAP_ID_MBMSDETACH:										 /* 46 */
	case RNSAP_ID_DIRECTINFORMATIONTRANSFER:						 /* 48 */
		offset = offset + (length<<3);
		break;
	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_rnsap_SuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_rnsap_successfulOutcomeValue, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_rnsap_initiatingMessageValue);

	offset = dissect_per_length_determinant(tvb, offset, pinfo, value_tree, hf_rnsap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){

	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_rnsap_UnsuccessfulOutcomeValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_rnsap_unsuccessfulOutcomeValue, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_rnsap_UnsuccessfulOutcomeValue);

	offset = dissect_per_length_determinant(tvb, offset, pinfo, value_tree, hf_rnsap_pdu_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProcedureCode){

	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset)
	return offset;
}

static int dissect_rnsap_ProtocolIEValueValue(tvbuff_t *tvb, int offset, packet_info *pinfo, proto_tree *tree){
	proto_item	*value_item = NULL;
	proto_tree	*value_tree = NULL;
	guint length;

    value_item = proto_tree_add_item(tree, hf_rnsap_value, tvb, 0, -1, FALSE);
    value_tree = proto_item_add_subtree(value_item, ett_rnsap_ProtocolIEValueValue);

	offset = dissect_per_length_determinant(tvb, offset, pinfo, value_tree, hf_rnsap_IE_length, &length);
	proto_item_set_len(value_item,length);

	
	switch(ProtocolIE_ID){

	case RNSAP_ID_ALLOWEDQUEUINGTIME:											/*  4 */
		offset = dissect_id_AllowedQueuingTime(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_ALLOWED_RATE_INFORMATION:										/*  42 */
		offset = dissect_id_Allowed_Rate_Information(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_ANTENNACOLOCATIONINDICATOR:									/*  309 */
		offset = dissect_id_AntennaColocationIndicator(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_BINDINGID:													/*  5 */
		offset = dissect_id_BindingID(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_C_ID:															/*  6 */
		offset = dissect_id_C_ID(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_C_RNTI:														/*  7 */
		offset = dissect_id_C_RNTI(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_CELL_CAPACITY_CLASS_VALUE:									/*  303 */
		offset = dissect_id_Cell_Capacity_Class_Value(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_CFN:															/*  8 */
		offset = dissect_id_CFN(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_CN_CS_DOMAINIDENTIFIER:										/*  9 */
		offset = dissect_id_CN_CS_DomainIdentifier(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_CN_PS_DOMAINIDENTIFIER:										/*  10 */
		offset = dissect_id_CN_PS_DomainIdentifier(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_CAUSE:														/*  11 */
		offset = dissect_id_Cause(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_COVERAGEINDICATOR:											/*  310 */
		offset = dissect_id_CoverageIndicator(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_CRITICALITYDIAGNOSTICS:										/*  20 */
		offset = dissect_id_CriticalityDiagnostics(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_CONTEXTINFOITEM_RESET:										/*  211 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_CONTEXTGROUPINFOITEM_RESET:									/*  515 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_D_RNTI:														/*  21 */
		offset = dissect_id_D_RNTI(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_D_RNTI_RELEASEINDICATION:										/*  22 */
		offset = dissect_id_D_RNTI_ReleaseIndication(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_DCHS_TO_ADD_FDD:												/*  26 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DCHS_TO_ADD_TDD:												/*  27 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPFDD:								/*  30 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFPREPTDD:								/*  31 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTFDD:								/*  32 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DCH_DELETELIST_RL_RECONFRQSTTDD:								/*  33 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DCH_FDD_INFORMATION:											/*  34 */
		offset = dissect_id_DCH_FDD_Information(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_DCH_TDD_INFORMATION:											/*  35 */
		offset = dissect_id_DCH_TDD_Information(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_FDD_DCHS_TO_MODIFY:											/*  39 */
	case RNSAP_ID_TDD_DCHS_TO_MODIFY:											/*  40 */
	case RNSAP_ID_DCH_INFORMATIONRESPONSE:										/*  43 */
	case RNSAP_ID_DCH_RATE_INFORMATIONITEM_RL_CONGESTIND:						/*  38 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONADDITEM_RL_RECONFPREPTDD:				/*  44 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD:				/*  45 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD:				/*  46 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD:					/*  47 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD:				/*  48 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:				/*  49 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD:					/*  50 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD:				/*  51 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD:				/*  52 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD:					/*  53 */
	case RNSAP_ID_FDD_DL_CODEINFORMATION:										/*  54 */
	case RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*  59 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATION_RL_SETUPRQSTFDD:							/*  60 */
		offset = dissect_id_DL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_DL_DPCH_INFORMATION_RL_RECONFRQSTFDD:							/*  61 */
	case RNSAP_ID_DL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD:					/*  62 */
	case RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD:					/*  63 */
	case RNSAP_ID_DL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD:						/*  64 */
	case RNSAP_ID_DL_DPCH_TIMINGADJUSTMENT:										/*  278 */
	case RNSAP_ID_DLREFERENCEPOWER:												/*  67 */
	case RNSAP_ID_DLREFERENCEPOWERLIST_DL_PC_RQST:								/*  68 */
	case RNSAP_ID_DL_REFERENCEPOWERINFORMATION_DL_PC_RQST:						/*  69 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_DPC_MODE:														/*  12 */
		offset = dissect_id_DPC_Mode(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_DRXCYCLELENGTHCOEFFICIENT:									/*  70 */
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL_IND:					/*  470 */
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_FAIL:						/*  471 */
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RPRT:						/*  71 */
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RQST:						/*  72 */
	case RNSAP_ID_DEDICATEDMEASUREMENTOBJECTTYPE_DM_RSP:						/*  73 */
	case RNSAP_ID_DEDICATEDMEASUREMENTTYPE:										/*  74 */
	case RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPFDD:			/*  82 */
	case RNSAP_ID_FACH_INFOFORUESELECTEDS_CCPCH_CTCH_RESOURCERSPTDD:			/*  83 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_GUARANTEED_RATE_INFORMATION:									/*  41 */
		offset = dissect_id_Guaranteed_Rate_Information(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_IMSI:															/*  84 */
		offset = dissect_id_IMSI(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_HCS_PRIO:														/*  311 */
		offset = dissect_id_HCS_Prio(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_L3_INFORMATION:												/*  85 */
		offset = dissect_id_L3_Information(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_ADJUSTMENTPERIOD:												/*  90 */
		offset = dissect_id_AdjustmentPeriod(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_MAXADJUSTMENTSTEP:											/*  91 */
	case RNSAP_ID_MEASUREMENTFILTERCOEFFICIENT:									/*  92 */
	case RNSAP_ID_MESSAGESTRUCTURE:												/*  57 */
	case RNSAP_ID_MEASUREMENTID:												/*  93 */
	case RNSAP_ID_NEIGHBOURING_GSM_CELLINFORMATION:								/*  13 */
	case RNSAP_ID_NEIGHBOURING_UMTS_CELLINFORMATIONITEM:						/*  95 */
	case RNSAP_ID_NRT_LOAD_INFORMATION_VALUE:									/*  305 */
	case RNSAP_ID_NRT_LOAD_INFORMATION_VALUE_INCRDECRTHRES:						/*  306 */
	case RNSAP_ID_PAGINGAREA_PAGINGRQST:										/*  102 */
	case RNSAP_ID_FACH_FLOWCONTROLINFORMATION:									/*  103 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_PARTIALREPORTINGINDICATOR:									/*  472 */
		offset = dissect_id_PartialReportingIndicator(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_PERMANENT_NAS_UE_IDENTITY:									/*  17 */
		offset = dissect_id_Permanent_NAS_UE_Identity(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_POWERADJUSTMENTTYPE:											/*  107 */
	case RNSAP_ID_RANAP_RELOCATIONINFORMATION:									/*  109 */
	case RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTFDD:							/*  110 */
	case RNSAP_ID_RL_INFORMATION_PHYCHRECONFRQSTTDD:							/*  111 */
	case RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTFDD:							/*  112 */
	case RNSAP_ID_RL_INFORMATION_RL_ADDITIONRQSTTDD:							/*  113 */
	case RNSAP_ID_RL_INFORMATION_RL_DELETIONRQST:								/*  114 */
	case RNSAP_ID_RL_INFORMATION_RL_FAILUREIND:									/*  115 */
	case RNSAP_ID_RL_INFORMATION_RL_RECONFPREPFDD:								/*  116 */
	case RNSAP_ID_RL_INFORMATION_RL_RESTOREIND:									/*  117 */
	case RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTFDD:								/*  118 */
	case RNSAP_ID_RL_INFORMATION_RL_SETUPRQSTTDD:								/*  119 */
	case RNSAP_ID_RL_INFORMATIONITEM_RL_CONGESTIND:								/*  55 */
	case RNSAP_ID_RL_INFORMATIONITEM_DM_RPRT:									/*  120 */
	case RNSAP_ID_RL_INFORMATIONITEM_DM_RQST:									/*  121 */
	case RNSAP_ID_RL_INFORMATIONITEM_DM_RSP:									/*  122 */
	case RNSAP_ID_RL_INFORMATIONITEM_RL_PREEMPTREQUIREDIND:						/*  2 */
	case RNSAP_ID_RL_INFORMATIONITEM_RL_SETUPRQSTFDD:							/*  123 */
	case RNSAP_ID_RL_INFORMATIONLIST_RL_CONGESTIND:								/*  56 */
	case RNSAP_ID_RL_INFORMATIONLIST_RL_ADDITIONRQSTFDD:						/*  124 */
	case RNSAP_ID_RL_INFORMATIONLIST_RL_DELETIONRQST:							/*  125 */
	case RNSAP_ID_RL_INFORMATIONLIST_RL_PREEMPTREQUIREDIND:						/*  1 */
	case RNSAP_ID_RL_INFORMATIONLIST_RL_RECONFPREPFDD:							/*  126 */
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD:						/*  127 */
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD:						/*  128 */
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_SETUPRSPTDD:						/*  129 */
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_ADDITIONRSPFDD:					/*  130 */
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFREADYFDD:					/*  131 */
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_RECONFRSPFDD:					/*  132 */
	case RNSAP_ID_RL_INFORMATIONRESPONSEITEM_RL_SETUPRSPFDD:					/*  133 */
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_ADDITIONRSPFDD:					/*  134 */
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFREADYFDD:					/*  135 */
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_RECONFRSPFDD:					/*  136 */
	case RNSAP_ID_RL_INFORMATIONRESPONSE_RL_RECONFRSPTDD:						/*  28 */
	case RNSAP_ID_RL_INFORMATIONRESPONSELIST_RL_SETUPRSPFDD:					/*  137 */
	case RNSAP_ID_RL_RECONFIGURATIONFAILURE_RL_RECONFFAIL:						/*  141 */
	case RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RPRT:								/*  143 */
	case RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RQST:								/*  144 */
	case RNSAP_ID_RL_SET_INFORMATIONITEM_DM_RSP:								/*  145 */
	case RNSAP_ID_RL_SET_INFORMATION_RL_FAILUREIND:								/*  146 */
	case RNSAP_ID_RL_SET_INFORMATION_RL_RESTOREIND:								/*  147 */
	case RNSAP_ID_RL_SET_SUCCESSFUL_INFORMATIONITEM_DM_FAIL:					/*  473 */
	case RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL:					/*  474 */
	case RNSAP_ID_RL_SET_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND:				/*  475 */
	case RNSAP_ID_RL_SUCCESSFUL_INFORMATIONITEM_DM_FAIL:						/*  476 */
	case RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL:						/*  477 */
	case RNSAP_ID_RL_UNSUCCESSFUL_INFORMATIONITEM_DM_FAIL_IND:					/*  478 */
	case RNSAP_ID_REPORTCHARACTERISTICS:										/*  152 */
	case RNSAP_ID_REPORTING_OBJECT_RL_FAILUREIND:								/*  153 */
	case RNSAP_ID_REPORING_OBJECT_RL_RESTOREIND:								/*  154 */
	case RNSAP_ID_RT_LOAD_VALUE:												/*  307 */
	case RNSAP_ID_RT_LOAD_VALUE_INCRDECRTHRES:									/*  308 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_S_RNTI:														/*  155 */
		offset = dissect_id_S_RNTI(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_RESETINDICATOR:												/*  244 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_RNC_ID:														/*  245 */
		offset = dissect_id_RNC_ID(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_SAI:															/*  156 */
		offset = dissect_id_SAI(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_SRNC_ID:														/*  157 */
		offset = dissect_id_SRNC_ID(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD:		/*  159 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_SUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD:			/*  160 */
	case RNSAP_ID_TRANSPORTBEARERID:											/*  163 */
	case RNSAP_ID_TRANSPORTBEARERREQUESTINDICATOR:								/*  164 */
	case RNSAP_ID_TRANSPORTLAYERADDRESS:										/*  165 */
	case RNSAP_ID_TYPEOFERROR:													/*  140 */
	case RNSAP_ID_UC_ID:														/*  166 */
	case RNSAP_ID_UL_CCTRCH_ADDINFORMATION_RL_RECONFPREPTDD:					/*  167 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONADDLIST_RL_RECONFPREPTDD:				/*  169 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_SETUPRQSTTDD:					/*  171 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_SETUPRQSTTDD:					/*  172 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_PHYCHRECONFRQSTTDD:				/*  173 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:				/*  174 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_RECONFREADYTDD:				/*  175 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLISTIE_RL_SETUPRSPTDD:					/*  176 */
	case RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*  177 */
	case RNSAP_ID_UL_DPCH_INFORMATION_RL_RECONFRQSTFDD:							/*  178 */
	case RNSAP_ID_UL_DPCH_INFORMATION_RL_SETUPRQSTFDD:							/*  179 */
		offset = dissect_id_UL_DPCH_Information_RL_SetupRqstFDD(tvb, offset, pinfo, value_tree);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONITEM_PHYCHRECONFRQSTTDD:					/*  180 */
		offset = offset + (length<<3);
		break;
	case RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_ADDITIONRSPTDD:					/*  181 */
	case RNSAP_ID_UL_DPCH_INFORMATIONITEM_RL_SETUPRSPTDD:						/*  182 */
	case RNSAP_ID_UL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:				/*  183 */
	case RNSAP_ID_UL_SIRTARGET:													/*  184 */
	case RNSAP_ID_URA_INFORMATION:												/*  185 */
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILUREFDD:		/*  188 */
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILUREFDD:		/*  189 */
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_SETUPFAILURETDD:		/*  190 */
	case RNSAP_ID_ACTIVE_PATTERN_SEQUENCE_INFORMATION:							/*  193 */
	case RNSAP_ID_ADJUSTMENTRATIO:												/*  194 */
	case RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILUREFDD:								/*  197 */
	case RNSAP_ID_CAUSELEVEL_RL_ADDITIONFAILURETDD:								/*  198 */
	case RNSAP_ID_CAUSELEVEL_RL_RECONFFAILURE:									/*  199 */
	case RNSAP_ID_CAUSELEVEL_RL_SETUPFAILUREFDD:								/*  200 */
	case RNSAP_ID_CAUSELEVEL_RL_SETUPFAILURETDD:								/*  201 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFPREPTDD:				/*  205 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFPREPTDD:				/*  206 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD:				/*  207 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD:				/*  208 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD:				/*  209 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD:				/*  210 */
	case RNSAP_ID_DL_DPCH_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:				/*  212 */
	case RNSAP_ID_DL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD:			/*  213 */
	case RNSAP_ID_DL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD:			/*  214 */
	case RNSAP_ID_DSCHS_TO_ADD_TDD:												/*  215 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_216:								/*  216 */
	case RNSAP_ID_DSCH_DELETELIST_RL_RECONFPREPTDD:								/*  217 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_218:								/*  218 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_219:								/*  219 */
	case RNSAP_ID_DSCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:						/*  220 */
	case RNSAP_ID_DSCH_INFORMATIONLISTIES_RL_SETUPRSPTDD:						/*  221 */
	case RNSAP_ID_DSCH_TDD_INFORMATION:											/*  222 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_223:								/*  223 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_226:								/*  226 */
	case RNSAP_ID_DSCH_MODIFYLIST_RL_RECONFPREPTDD:								/*  227 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_228:								/*  228 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_324:								/*  324 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_229:								/*  229 */
	case RNSAP_ID_DSCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD:				/*  230 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_29:								/*  29 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_225:								/*  225 */
	case RNSAP_ID_GA_CELL:														/*  232 */
	case RNSAP_ID_GA_CELLADDITIONALSHAPES:										/*  3 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_246:								/*  246 */
	case RNSAP_ID_TRANSMISSION_GAP_PATTERN_SEQUENCE_INFORMATION:				/*  255 */
	case RNSAP_ID_UL_CCTRCH_DELETEINFORMATION_RL_RECONFPREPTDD:					/*  256 */
	case RNSAP_ID_UL_CCTRCH_MODIFYINFORMATION_RL_RECONFPREPTDD:					/*  257 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYITEM_RL_RECONFRQSTTDD:				/*  258 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFPREPTDD:				/*  259 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFPREPTDD:				/*  260 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONMODIFYLIST_RL_RECONFRQSTTDD:				/*  261 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONDELETEITEM_RL_RECONFRQSTTDD:				/*  262 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONDELETELIST_RL_RECONFRQSTTDD:				/*  263 */
	case RNSAP_ID_UL_DPCH_INFORMATIONDELETELISTIE_RL_RECONFREADYTDD:			/*  264 */
	case RNSAP_ID_UL_DPCH_INFORMATIONMODIFYLISTIE_RL_RECONFREADYTDD:			/*  265 */
	case RNSAP_ID_UNSUCCESSFULRL_INFORMATIONRESPONSE_RL_ADDITIONFAILURETDD:		/*  266 */
	case RNSAP_ID_USCHS_TO_ADD:													/*  267 */
	case RNSAP_ID_USCH_DELETELIST_RL_RECONFPREPTDD:								/*  268 */
	case RNSAP_ID_USCH_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:						/*  269 */
	case RNSAP_ID_USCH_INFORMATIONLISTIES_RL_SETUPRSPTDD:						/*  270 */
	case RNSAP_ID_USCH_INFORMATION:												/*  271 */
	case RNSAP_ID_USCH_MODIFYLIST_RL_RECONFPREPTDD:								/*  272 */
	case RNSAP_ID_USCHTOBEADDEDORMODIFIEDLIST_RL_RECONFREADYTDD:				/*  273 */
	case RNSAP_ID_DL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD:				/*  274 */
	case RNSAP_ID_UL_PHYSICAL_CHANNEL_INFORMATION_RL_SETUPRQSTTDD:				/*  275 */
	case RNSAP_ID_CLOSEDLOOPMODE1_SUPPORTINDICATOR:								/*  276 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_277:								/*  277 */
	case RNSAP_ID_STTD_SUPPORTINDICATOR:										/*  279 */
	case RNSAP_ID_CFNREPORTINGINDICATOR:   										/*  14 */
	case RNSAP_ID_CNORIGINATEDPAGE_PAGINGRQST:									/*  23 */
	case RNSAP_ID_INNERLOOPDLPCSTATUS:											/*  24 */
	case RNSAP_ID_PROPAGATIONDELAY:												/*  25 */
	case RNSAP_ID_RXTIMINGDEVIATIONFORTA:										/*  36 */
	case RNSAP_ID_TIMESLOT_ISCP:												/*  37 */
	case RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_FAILUREIND:							/*  15 */
	case RNSAP_ID_CCTRCH_INFORMATIONITEM_RL_RESTOREIND:							/*  16 */
	case RNSAP_ID_COMMONMEASUREMENTACCURACY:									/*  280 */
	case RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RPRT:							/*  281 */
	case RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RQST:							/*  282 */
	case RNSAP_ID_COMMONMEASUREMENTOBJECTTYPE_CM_RSP:							/*  283 */
	case RNSAP_ID_COMMONMEASUREMENTTYPE:										/*  284 */
	case RNSAP_ID_CONGESTIONCAUSE:												/*  18 */
	case RNSAP_ID_SFN:															/*  285 */
	case RNSAP_ID_SFNREPORTINGINDICATOR:										/*  286 */
	case RNSAP_ID_INFORMATIONEXCHANGEID:										/*  287 */
	case RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RPRT:						/*  288 */
	case RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RQST:						/*  289 */
	case RNSAP_ID_INFORMATIONEXCHANGEOBJECTTYPE_INFEX_RSP:						/*  290 */
	case RNSAP_ID_INFORMATIONREPORTCHARACTERISTICS:								/*  291 */
	case RNSAP_ID_INFORMATIONTYPE:												/*  292 */
	case RNSAP_ID_NEIGHBOURING_LCR_TDD_CELLINFORMATION:							/*  58 */
	case RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_SETUPRQSTTDD:				/*  65 */
	case RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_SETUPRSPTDD:					/*  66 */
	case RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD:				/*  75 */
	case RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD:					/*  76 */
	case RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_SETUPRSPTDD:				/*  77 */
	case RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD:					/*  78 */
	case RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD:					/*  79 */
	case RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_SETUPRSPTDD:					/*  80 */
	case RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_ADDITIONRQSTTDD:			/*  81 */
	case RNSAP_ID_RL_LCR_INFORMATIONRESPONSE_RL_ADDITIONRSPTDD:					/*  86 */
	case RNSAP_ID_UL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:			/*  87 */
	case RNSAP_ID_UL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD:				/*  88 */
	case RNSAP_ID_DL_CCTRCH_LCR_INFORMATIONLISTIE_RL_ADDITIONRSPTDD:			/*  89 */
	case RNSAP_ID_DL_DPCH_LCR_INFORMATIONITEM_RL_ADDITIONRSPTDD:				/*  94 */
	case RNSAP_ID_DSCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD:				/*  96 */
	case RNSAP_ID_USCH_LCR_INFORMATIONLISTIES_RL_ADDITIONRSPTDD:				/*  97 */
	case RNSAP_ID_UL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:			/*  98 */
	case RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD:		/*  100 */
	case RNSAP_ID_DL_DPCH_LCR_INFORMATIONADDLISTIE_RL_RECONFREADYTDD:			/*  101 */
	case RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONMODIFYLIST_RL_RECONFREADYTDD:		/*  104 */
	case RNSAP_ID_UL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD:			/*  105 */
	case RNSAP_ID_DL_TIMESLOT_LCR_INFORMATIONLIST_PHYCHRECONFRQSTTDD:			/*  106 */
	case RNSAP_ID_TIMESLOT_ISCP_LCR_LIST_DL_PC_RQST_TDD:						/*  138 */
	case RNSAP_ID_TSTD_SUPPORT_INDICATOR_RL_SETUPRQSTTDD:						/*  139 */
	case RNSAP_ID_RESTRICTIONSTATEINDICATOR:									/*  142 */
	case RNSAP_ID_LOAD_VALUE:													/*  233 */
	case RNSAP_ID_LOAD_VALUE_INCRDECRTHRES:										/*  234 */
	case RNSAP_ID_ONMODIFICATION:												/*  235 */
	case RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE:							/*  236 */
	case RNSAP_ID_RECEIVED_TOTAL_WIDEBAND_POWER_VALUE_INCRDECRTHRES:			/*  237 */
	case RNSAP_ID_SFNSFNMEASUREMENTTHRESHOLDINFORMATION:						/*  238 */
	case RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE:								/*  239 */
	case RNSAP_ID_TRANSMITTED_CARRIER_POWER_VALUE_INCRDECRTHRES:				/*  240 */
	case RNSAP_ID_TUTRANGPSMEASUREMENTTHRESHOLDINFORMATION:						/*  241 */
	case RNSAP_ID_UL_TIMESLOT_ISCP_VALUE:										/*  242 */
	case RNSAP_ID_UL_TIMESLOT_ISCP_VALUE_INCRDECRTHRES:							/*  243 */
	case RNSAP_ID_RX_TIMING_DEVIATION_VALUE_LCR:								/*  293 */
	case RNSAP_ID_DPC_MODE_CHANGE_SUPPORTINDICATOR:								/*  19 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_247:								/*  247 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_295:								/*  295 */
	case RNSAP_ID_PRIMARYCCPCH_RSCP_RL_RECONFPREPTDD:							/*  202 */
	case RNSAP_ID_DL_TIMESLOT_ISCP_INFO_RL_RECONFPREPTDD:						/*  203 */
	case RNSAP_ID_DL_TIMESLOT_ISCP_LCR_INFORMATION_RL_RECONFPREPTDD:			/*  204 */
	case RNSAP_ID_DSCH_RNTI:													/*  249 */
	case RNSAP_ID_DL_POWERBALANCING_INFORMATION:								/*  296 */
	case RNSAP_ID_DL_POWERBALANCING_ACTIVATIONINDICATOR:						/*  297 */
	case RNSAP_ID_DL_POWERBALANCING_UPDATEDINDICATOR:							/*  298 */
	case RNSAP_ID_DL_REFERENCEPOWERINFORMATION:									/*  299 */
	case RNSAP_ID_ENHANCED_PRIMARYCPICH_ECNO:									/*  224 */
	case RNSAP_ID_IPDL_TDD_PARAMETERSLCR:										/*  252 */
	case RNSAP_ID_CELLCAPABILITYCONTAINER_FDD:									/*  300 */
	case RNSAP_ID_CELLCAPABILITYCONTAINER_TDD:									/*  301 */
	case RNSAP_ID_CELLCAPABILITYCONTAINER_TDD_LCR:								/*  302 */
	case RNSAP_ID_RL_SPECIFIC_DCH_INFO:											/*  317 */
	case RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATIONLIST:				/*  318 */
	case RNSAP_ID_RL_RECONFIGURATIONREQUESTFDD_RL_INFORMATION_IES:				/*  319 */
	case RNSAP_ID_RL_RECONFIGURATIONREQUESTTDD_RL_INFORMATION:					/*  321 */
	case RNSAP_ID_COMMONTRANSPORTCHANNELRESOURCESINITIALISATIONNOTREQUIRED:		/*  250 */
	case RNSAP_ID_DELAYEDACTIVATION:											/*  312 */
	case RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDFDD:					/*  313 */
	case RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDFDD:				/*  314 */
	case RNSAP_ID_DELAYEDACTIVATIONLIST_RL_ACTIVATIONCMDTDD:					/*  315 */
	case RNSAP_ID_DELAYEDACTIVATIONINFORMATION_RL_ACTIVATIONCMDTDD:				/*  316 */
	case RNSAP_ID_NEIGHBOURINGTDDCELLMEASUREMENTINFORMATIONLCR:					/*  251 */
	case RNSAP_ID_UL_SIR_TARGET_CCTRCH_INFORMATIONITEM_RL_SETUPRSPTDD:			/*  150 */
	case RNSAP_ID_UL_SIR_TARGET_CCTRCH_LCR_INFORMATIONITEM_RL_SETUPRSPTDD:		/*  151 */
	case RNSAP_ID_PRIMCCPCH_RSCP_DL_PC_RQSTTDD:									/*  451 */
	case RNSAP_ID_HSDSCH_FDD_INFORMATION:										/*  452 */
	case RNSAP_ID_HSDSCH_FDD_INFORMATION_RESPONSE:								/*  453 */
	case RNSAP_ID_HSDSCH_FDD_UPDATE_INFORMATION:								/*  466 */
	case RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY:									/*  456 */
	case RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND:	/*  516 */
	case RNSAP_ID_HSDSCHMACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND:	/*  517 */
	case RNSAP_ID_HSDSCH_RNTI:													/*  457 */
	case RNSAP_ID_HSDSCH_TDD_INFORMATION:										/*  458 */
	case RNSAP_ID_HSDSCH_TDD_INFORMATION_RESPONSE:								/*  459 */
	case RNSAP_ID_HSDSCH_TDD_UPDATE_INFORMATION:								/*  467 */
	case RNSAP_ID_HSPDSCH_RL_ID:												/*  463 */
	case RNSAP_ID_HSDSCH_MACDFLOWS_TO_ADD:										/*  531 */
	case RNSAP_ID_HSDSCH_MACDFLOWS_TO_DELETE:									/*  532 */
	case RNSAP_ID_ANGLE_OF_ARRIVAL_VALUE_LCR:									/*  148 */
	case RNSAP_ID_TRAFFICCLASS:													/*  158 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_248:								/*  248 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_253:								/*  253 */
	case RNSAP_ID_PDSCH_RL_ID:													/*  323 */
	case RNSAP_ID_TIMESLOT_RL_SETUPRSPTDD:										/*  325 */
	case RNSAP_ID_GERAN_CELL_CAPABILITY:										/*  468 */
	case RNSAP_ID_GERAN_CLASSMARK:												/*  469 */
	case RNSAP_ID_DSCH_INITIALWINDOWSIZE:										/*  480 */
	case RNSAP_ID_UL_SYNCHRONISATION_PARAMETERS_LCR:							/*  464 */
	case RNSAP_ID_SNA_INFORMATION:												/*  479 */
	case RNSAP_ID_MACHS_RESETINDICATOR:											/*  465 */
	case RNSAP_ID_TDD_DL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD:	/*  481 */
	case RNSAP_ID_TDD_UL_DPCH_TIMESLOTFORMATMODIFYITEM_LCR_RL_RECONFREADYTDD:	/*  482 */
	case RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_LCR_RL_SETUPRQSTTDD:					/*  483 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD:					/*  484 */
	case RNSAP_ID_UL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD:					/*  485 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_ADDITIONRQSTTDD:					/*  486 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONITEM_RL_ADDITIONRQSTTDD:					/*  487 */
	case RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONADD_LCR_RL_RECONFPREPTDD:	/*  488 */
	case RNSAP_ID_TDD_TPC_UPLINKSTEPSIZE_INFORMATIONMODIFY_LCR_RL_RECONFPREPTDD:/*  489 */
	case RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONADD_RL_RECONFPREPTDD:		/*  490 */
	case RNSAP_ID_TDD_TPC_DOWNLINKSTEPSIZE_INFORMATIONMODIFY_RL_RECONFPREPTDD:	/*  491 */
	case RNSAP_ID_UL_TIMINGADVANCECTRL_LCR:										/*  492 */
	case RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLIST_PHYCHRECONFRQSTTDD:			/*  493 */
	case RNSAP_ID_HSPDSCH_TIMESLOT_INFORMATIONLISTLCR_PHYCHRECONFRQSTTDD:		/*  494 */
	case RNSAP_ID_HS_SICH_RECEPTION_QUALITY: 									/*  495 */
	case RNSAP_ID_HS_SICH_RECEPTION_QUALITY_MEASUREMENT_VALUE:					/*  496 */
	case RNSAP_ID_HSSICH_INFO_DM_RPRT:											/*  497 */
	case RNSAP_ID_HSSICH_INFO_DM_RQST:											/*  498 */
	case RNSAP_ID_HSSICH_INFO_DM:												/*  499 */
	case RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_SETUPRSPTDD:						/*  500 */
	case RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_SETUPRSPTDD:						/*  501 */
	case RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_ADDITIONRSPTDD:					/*  502 */
	case RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_ADDITIONRSPTDD:					/*  503 */
	case RNSAP_ID_CCTRCH_MAXIMUM_DL_POWER_RL_RECONFREADYTDD:					/*  504 */
	case RNSAP_ID_CCTRCH_MINIMUM_DL_POWER_RL_RECONFREADYTDD:					/*  505 */
	case RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD:	/*  506 */
	case RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONMODIFYITEM_RL_RECONFREADYTDD:	/*  507 */
	case RNSAP_ID_DL_CCTRCH_INFORMATIONLIST_RL_RECONFRSPTDD:					/*  508 */
	case RNSAP_ID_DL_DPCH_INFORMATIONMODIFYITEM_LCR_RL_RECONFRSPTDD:			/*  509 */
	case RNSAP_ID_MAXIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM:					/*  510 */
	case RNSAP_ID_MINIMUM_DL_POWER_TIMESLOTLCR_INFORMATIONITEM:					/*  511 */
	case RNSAP_ID_TDD_SUPPORT_8PSK:												/*  512 */
	case RNSAP_ID_TDD_MAXNRDLPHYSICALCHANNELS:									/*  513 */
	case RNSAP_ID_EXTENDEDGSMCELLINDIVIDUALOFFSET:								/*  514 */
	case RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATIONLIST:			/*  518 */
	case RNSAP_ID_PRIMARY_CPICH_USAGE_FOR_CHANNEL_ESTIMATION:					/*  519 */
	case RNSAP_ID_SECONDARY_CPICH_INFORMATION:									/*  520 */
	case RNSAP_ID_SECONDARY_CPICH_INFORMATION_CHANGE:							/*  521 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_522:								/*  522 */
	case RNSAP_ID_UNUSED_PROTOCOLIE_RNSAP_ID_523:								/*  523 */
	case RNSAP_ID_RL_PARAMETERUPDATEINDICATIONFDD_RL_INFORMATION_ITEM:			/*  524 */
	case RNSAP_ID_PHASE_REFERENCE_UPDATE_INDICATOR:								/*  525 */
	case RNSAP_ID_UNIDIRECTIONAL_DCH_INDICATOR:									/*  526 */
	case RNSAP_ID_RL_INFORMATION_RL_RECONFPREPTDD:								/*  527 */
	case RNSAP_ID_MULTIPLE_RL_INFORMATIONRESPONSE_RL_RECONFREADYTDD:			/*  528 */
	case RNSAP_ID_RL_RECONFIGURATIONRESPONSETDD_RL_INFORMATION:					/*  529 */
	case RNSAP_ID_SATELLITE_ALMANAC_INFORMATION_EXTITEM:						/*  530 */
	case RNSAP_ID_HSDSCH_INFORMATION_TO_MODIFY_UNSYNCHRONISED:					/*  533 */
	case RNSAP_ID_TNLQOS:														/*  534 */
	case RNSAP_ID_RTLOADVALUE:													/*  535 */
	case RNSAP_ID_NRTLOADINFORMATIONVALUE:										/*  536 */
	case RNSAP_ID_CELLPORTIONID:												/*  537 */
	case RNSAP_ID_UPPTSINTERFERENCEVALUE:										/*  538 */
	case RNSAP_ID_PRIMARYCCPCH_RSCP_DELTA:										/*  539 */
	case RNSAP_ID_UEMEASUREMENTTYPE:											/*  540 */
	case RNSAP_ID_UEMEASUREMENTTIMESLOTINFOHCR:									/*  541 */
	case RNSAP_ID_UEMEASUREMENTTIMESLOTINFOLCR:									/*  542 */
	case RNSAP_ID_UEMEASUREMENTREPORTCHARACTERISTICS:							/*  543 */
	case RNSAP_ID_UEMEASUREMENTPARAMETERMODALLOW:								/*  544 */
	case RNSAP_ID_UEMEASUREMENTVALUEINFORMATION:								/*  545 */
	case RNSAP_ID_INTERFACESTOTRACEITEM:										/*  546 */
	case RNSAP_ID_LISTOFINTERFACESTOTRACE:										/*  547 */
	case RNSAP_ID_TRACEDEPTH:													/*  548 */
	case RNSAP_ID_TRACERECORDINGSESSIONREFERENCE:								/*  549 */
	case RNSAP_ID_TRACEREFERENCE:												/*  550 */
	case RNSAP_ID_UEIDENTITY:													/*  551 */
	case RNSAP_ID_NACC_RELATED_DATA:											/*  552 */
	case RNSAP_ID_GSM_CELL_INFEX_RQST:											/*  553 */
	case RNSAP_ID_MEASUREMENTRECOVERYBEHAVIOR:									/*  554 */
	case RNSAP_ID_MEASUREMENTRECOVERYREPORTINGINDICATOR:						/*  555 */
	case RNSAP_ID_MEASUREMENTRECOVERYSUPPORTINDICATOR:							/*  556 */
	case RNSAP_ID_DL_DPCH_POWER_INFORMATION_RL_RECONFPREPFDD:					/*  557 */
	case RNSAP_ID_F_DPCH_INFORMATION_RL_RECONFPREPFDD:							/*  558 */
	case RNSAP_ID_F_DPCH_INFORMATION_RL_SETUPRQSTFDD:							/*  559 */
	case RNSAP_ID_MBMS_BEARER_SERVICE_LIST:										/*  560 */
	case RNSAP_ID_MBMS_BEARER_SERVICE_LIST_INFEX_RSP:							/*  561 */
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD:								/*  562 */
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD:								/*  563 */
	case RNSAP_ID_OLD_URA_ID:													/*  564 */
	case RNSAP_ID_UE_STATE:														/*  568 */
	case RNSAP_ID_URA_ID:														/*  569 */
	case RNSAP_ID_HARQ_PREAMBLE_MODE:											/*  571 */
	case RNSAP_ID_SYNCHRONISATIONINDICATOR:										/*  572 */
	case RNSAP_ID_UL_DPDCHINDICATOREDCH:										/*  573 */
	case RNSAP_ID_EDPCH_INFORMATION:											/*  574 */
	case RNSAP_ID_RL_SPECIFIC_EDCH_INFORMATION:									/*  575 */
	case RNSAP_ID_EDCH_RL_INDICATION:											/*  576 */
	case RNSAP_ID_EDCH_FDD_INFORMATION:											/*  577 */
	case RNSAP_ID_EDCH_RLSET_ID:												/*  578 */
	case RNSAP_ID_SERVING_EDCHRL_ID:											/*  579 */
	case RNSAP_ID_EDCH_FDD_DL_CONTROLCHANNELINFORMATION:						/*  580 */
	case RNSAP_ID_EDCH_FDD_INFORMATIONRESPONSE:									/*  581 */
	case RNSAP_ID_EDCH_MACDFLOWS_TO_ADD:										/*  582 */
	case RNSAP_ID_EDCH_FDD_INFORMATION_TO_MODIFY:								/*  583 */
	case RNSAP_ID_EDCH_MACDFLOWS_TO_DELETE:										/*  584 */
	case RNSAP_ID_EDPCH_INFORMATION_RLRECONFREQUEST_FDD:						/*  585 */
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_PREEMPTREQUIREDIND:	/*  586 */
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_PREEMPTREQUIREDIND:	/*  587 */
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONLIST_RL_CONGESTIND:			/*  588 */
	case RNSAP_ID_EDCH_MACDFLOWSPECIFICINFORMATIONITEM_RL_CONGESTIND:			/*  589 */
	case RNSAP_ID_MBMS_BEARER_SERVICE_FULL_ADDRESS:								/*  590 */
	case RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT:								/*  591 */
	case RNSAP_ID_INITIAL_DL_DPCH_TIMINGADJUSTMENT_ALLOWED:						/*  592 */
	case RNSAP_ID_USER_PLANE_CONGESTION_FIELDS_INCLUSION:						/*  593 */
	case RNSAP_ID_HARQ_PREAMBLE_MODE_ACTIVATION_INDICATOR:						/*  594 */
	case RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_TDD_DM_RSP:			/*  595 */
	case RNSAP_ID_MULTIPLE_DEDICATEDMEASUREMENTVALUELIST_LCR_TDD_DM_RSP:		/*  596 */
	case RNSAP_ID_PROVIDEDINFORMATION:											/*  597 */
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICEFDD_PFL:							/*  598 */
	case RNSAP_ID_ACTIVE_MBMS_BEARER_SERVICETDD_PFL:							/*  599 */
	case RNSAP_ID_FREQUENCYBANDINDICATOR:										/*  600	*/
		offset = offset + (length<<3);
		break;
	default:
		offset = offset + (length<<3);
		break;
	}
	BYTE_ALIGN_OFFSET(offset);	
	return offset;
}
static void
dissect_rnsap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	proto_item	*rnsap_item = NULL;
	proto_tree	*rnsap_tree = NULL;
	int			offset = 0;

	top_tree = tree;

	/* make entry in the Protocol column on summary display */
	if (check_col(pinfo->cinfo, COL_PROTOCOL))
		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RNSAP");

    /* create the rnsap protocol tree */
    rnsap_item = proto_tree_add_item(tree, proto_rnsap, tvb, 0, -1, FALSE);
    rnsap_tree = proto_item_add_subtree(rnsap_item, ett_rnsap);

	dissect_RNSAP_PDU_PDU(tvb, pinfo, rnsap_tree);
}
/*--- proto_register_rnsap -------------------------------------------*/
void proto_register_rnsap(void) {

  /* List of fields */

  static hf_register_info hf[] = {
	{ &hf_rnsap_pdu_length,
		{ "PDU Length", "rnsap.pdu_length", FT_UINT32, BASE_DEC,
		NULL, 0, "Number of octets in the PDU", HFILL }},
	{ &hf_rnsap_IE_length,
		{ "IE Length", "rnsap.ie_length", FT_UINT32, BASE_DEC,
		NULL, 0, "Number of octets in the IE", HFILL }},
#include "packet-rnsap-hfarr.c"
  };

  /* List of subtrees */
  static gint *ett[] = {
		  &ett_rnsap,
		  &ett_rnsap_initiatingMessageValue,
		  &ett_rnsap_ProtocolIEValueValue,
		  &ett_rnsap_SuccessfulOutcomeValue,
		  &ett_rnsap_UnsuccessfulOutcomeValue,
#include "packet-rnsap-ettarr.c"
  };


  /* Register protocol */
  proto_rnsap = proto_register_protocol(PNAME, PSNAME, PFNAME);
  /* Register fields and subtrees */
  proto_register_field_array(proto_rnsap, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));

 
  register_dissector("rnsap", dissect_rnsap, proto_rnsap);


}


/*--- proto_reg_handoff_rnsap ---------------------------------------*/
void
proto_reg_handoff_rnsap(void)
{

	rnsap_handle = find_dissector("rnsap");

}


